require 'faraday'
require 'sawyer'

module Chairman
  module Holmes
    class Client
      attr_accessor :headers, :body

      def initialize(host)
        @api_client = Sawyer::Agent.new(host, {serializer: Sawyer::Serializer.yajl}) do |http|
          http.headers['content-type'] = 'application/json'
        end

        @headers = {}
        @body = ''
      end

      def root
        get '/'
      end

      def extensions
        get '/extensions'
      end

      private

        def passthrough_headers
          [
            'content-type',
            'last-modified',
            'cache-control'
          ]
        end

        def get(path, params = {})
          call @api_client.call :get, path, params
        end

        def post(path, params = {})
          call @api_client.call :post, path, params
        end

        def call(api_response)
          api_response.headers.each do |key, value|
            if passthrough_headers.include? key
              @headers[key] = value
            end
          end

          if api_response.data.respond_to? :to_hash
            @body = api_response.data.to_attrs.to_json
          else
            @body = api_response.data
          end

          return api_response.status
        end
    end

  

    class Routes < Sinatra::Base
      set :protection, except: :frame_options

      get '/authorize' do

        redirect to Chairman.session.authorize_url(Chairman.client_id, scope: Chairman.scope.join(','))
      end

      get '/authorize/return' do
        halt if params[:error] == 'access_denied'

        token = Chairman.session.exchange_code_for_token(params[:code], Chairman.client_id, Chairman.client_secret, {accept: 'application/json'})

        # TODO: Verify that returned scope is the same as we requested
        # See: http://developer.github.com/changes/2013-10-04-oauth-changes-coming/

        @user = Chairman.session(token.access_token).user

        session[:github_token] = token.access_token
        session[:github_id] = @user.login
      end

      get '/logout' do
        session[:github_token] = nil
        session[:github_id] = nil
      end
    end

  end
end

